"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var socket_1 = require("mudb/socket");
var mustreams_1 = require("mustreams");
var MuBufferWrapper = (function () {
    function MuBufferWrapper(bytes) {
        this._buffer = mustreams_1.allocBuffer(bytes.length);
        this.bytes = this._buffer.uint8.subarray(0, bytes.length);
        this.bytes.set(bytes);
    }
    MuBufferWrapper.prototype.free = function () {
        mustreams_1.freeBuffer(this._buffer);
    };
    return MuBufferWrapper;
}());
function calcDelay(latency, jitter) {
    return latency + Math.floor(Math.random() * jitter);
}
function drain(pendingMessages, handle) {
    var message = pendingMessages.shift();
    if (typeof message === 'string') {
        handle(message, false);
    }
    else if (message) {
        handle(message.bytes, false);
        message.free();
    }
}
var MuDebugSocket = (function () {
    function MuDebugSocket(spec) {
        this.state = socket_1.MuSocketState.INIT;
        this.inLatency = 0;
        this.inJitter = 0;
        this.inPacketLoss = 0;
        this.outLatency = 0;
        this.outJitter = 0;
        this.outPacketLoss = 0;
        this._inbox = [];
        this._outbox = [];
        this.socket = spec.socket;
        this.sessionId = this.socket.sessionId;
        if (typeof spec.inLatency === 'number') {
            this.inLatency = Math.max(0, spec.inLatency);
        }
        if (typeof spec.inJitter === 'number') {
            this.inJitter = Math.max(0, spec.inJitter);
        }
        if (typeof spec.inPacketLoss === 'number') {
            this.inPacketLoss = Math.min(100, Math.max(0, spec.inPacketLoss));
        }
        if (typeof spec.outLatency === 'number') {
            this.outLatency = Math.max(0, spec.outLatency);
        }
        if (typeof spec.outJitter === 'number') {
            this.outJitter = Math.max(0, spec.outJitter);
        }
        if (typeof spec.outPacketLoss === 'number') {
            this.outPacketLoss = Math.min(100, Math.max(0, spec.outPacketLoss));
        }
    }
    MuDebugSocket.prototype.open = function (spec) {
        var _this = this;
        this.socket.open({
            ready: spec.ready,
            message: function (data, unreliable) {
                if (Math.random() * 100 <= _this.inPacketLoss) {
                    return;
                }
                if (unreliable) {
                    setTimeout(function () { return spec.message(data, true); }, calcDelay(_this.inLatency, _this.inJitter));
                }
                else {
                    var message = typeof data === 'string' ? data : new MuBufferWrapper(data);
                    _this._inbox.push(message);
                    setTimeout(function () { return drain(_this._inbox, spec.message); }, calcDelay(_this.inLatency, _this.inJitter));
                }
            },
            close: spec.close,
        });
    };
    MuDebugSocket.prototype.send = function (data, unreliable) {
        var _this = this;
        if (Math.random() * 100 <= this.outPacketLoss) {
            return;
        }
        if (unreliable) {
            setTimeout(function () { return _this.socket.send(data, true); }, calcDelay(this.outLatency, this.outJitter));
        }
        else {
            var message = typeof data === 'string' ? data : new MuBufferWrapper(data);
            this._outbox.push(message);
            setTimeout(function () { return drain(_this._outbox, function (data_, unreliable_) { return _this.socket.send(data_, unreliable_); }); }, calcDelay(this.outLatency, this.outJitter));
        }
    };
    MuDebugSocket.prototype.close = function () {
        this.socket.close();
    };
    return MuDebugSocket;
}());
exports.MuDebugSocket = MuDebugSocket;
var MuDebugServer = (function () {
    function MuDebugServer(spec) {
        this.state = socket_1.MuSocketServerState.INIT;
        this.clients = [];
        this.socketServer = spec.socketServer;
        this.inLatency = spec.inLatency || 0;
        this.inJitter = spec.inJitter || 0;
        this.inPacketLoss = spec.inPacketLoss || 0;
        this.outLatency = spec.outLatency || 0;
        this.outJitter = spec.outJitter || 0;
        this.outPacketLoss = spec.outPacketLoss || 0;
    }
    MuDebugServer.prototype.start = function (spec) {
        var _this = this;
        this.socketServer.start({
            ready: spec.ready,
            connection: function (socket) {
                var client = new MuDebugSocket({
                    socket: socket,
                    inLatency: _this.inLatency,
                    inJitter: _this.inJitter,
                    inPacketLoss: _this.inPacketLoss,
                    outLatency: _this.outLatency,
                    outJitter: _this.outJitter,
                    outPacketLoss: _this.outPacketLoss,
                });
                _this.clients.push(client);
                spec.connection(client);
            },
            close: spec.close,
        });
    };
    MuDebugServer.prototype.close = function () {
        this.socketServer.close();
    };
    return MuDebugServer;
}());
exports.MuDebugServer = MuDebugServer;
//# sourceMappingURL=index.js.map