"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var test = require("tape");
var mulocal_socket_1 = require("mulocal-socket");
var index_1 = require("../index");
var mudb_1 = require("mudb");
var schema_1 = require("./schema");
var PING_OPCODE = 0x9;
var PONG_OPCODE = 0xA;
function randomId() {
    return Math.random().toString(36).substr(2);
}
test('LocalSocket - simulating network latency on client side', function (t) {
    t.plan(3);
    var timestamp;
    var upstreamLatency = 75;
    var downstreamLatency = 25;
    var socketServer = mulocal_socket_1.createLocalSocketServer();
    var muServer = new mudb_1.MuServer(socketServer);
    var serverProtocol = muServer.protocol(schema_1.protocolSchema);
    serverProtocol.configure({
        message: {
            ping: function (client, data) {
                if (data === PING_OPCODE) {
                    var delay = Date.now() - timestamp;
                    t.ok(delay > upstreamLatency, "server should receive opcode after >" + upstreamLatency + " ms");
                    client.message.pong(PONG_OPCODE);
                }
            },
        },
    });
    muServer.start({
        ready: function () {
            var socket = mulocal_socket_1.createLocalSocket({
                sessionId: randomId(),
                server: socketServer,
            });
            var debugSocket = new index_1.MuDebugSocket({
                socket: socket,
                outLatency: upstreamLatency,
                inLatency: downstreamLatency,
            });
            var muClient = new mudb_1.MuClient(debugSocket);
            var clientProtocol = muClient.protocol(schema_1.protocolSchema);
            clientProtocol.configure({
                message: {
                    pong: function (data) {
                        var delay = Date.now() - timestamp;
                        var latency = upstreamLatency + downstreamLatency;
                        t.equal(data, PONG_OPCODE);
                        t.ok(delay > latency, "client should receive opcode after >" + latency + " ms");
                    },
                },
            });
            muClient.start({
                ready: function () {
                    clientProtocol.server.message.ping(PING_OPCODE);
                    timestamp = Date.now();
                },
            });
        },
    });
});
test('LocalSocket - simulating network latency on server side', function (t) {
    t.plan(3);
    var timestamp;
    var upstreamLatency = 75;
    var downstreamLatency = 25;
    var socketServer = mulocal_socket_1.createLocalSocketServer();
    var debugServer = new index_1.MuDebugServer({
        socketServer: socketServer,
        inLatency: upstreamLatency,
        outLatency: downstreamLatency,
    });
    var muServer = new mudb_1.MuServer(debugServer);
    var serverProtocol = muServer.protocol(schema_1.protocolSchema);
    serverProtocol.configure({
        message: {
            ping: function (client, data) {
                if (data === PING_OPCODE) {
                    var delay = Date.now() - timestamp;
                    t.ok(delay > upstreamLatency, "server should receive opcode after >" + upstreamLatency);
                    client.message.pong(PONG_OPCODE);
                }
            },
        },
    });
    muServer.start({
        ready: function () {
            var socket = mulocal_socket_1.createLocalSocket({
                sessionId: randomId(),
                server: socketServer,
            });
            var muClient = new mudb_1.MuClient(socket);
            var clientProtocol = muClient.protocol(schema_1.protocolSchema);
            clientProtocol.configure({
                message: {
                    pong: function (data) {
                        var delay = Date.now() - timestamp;
                        var latency = upstreamLatency + downstreamLatency;
                        t.equal(data, PONG_OPCODE);
                        t.ok(delay > latency, "client should receive opcode after >" + latency + " ms");
                    },
                },
            });
            muClient.start({
                ready: function () {
                    clientProtocol.server.message.ping(PING_OPCODE);
                    timestamp = Date.now();
                },
            });
        },
    });
});
test('LocalSocket - simulating network latency on both sides', function (t) {
    t.plan(3);
    var timestamp;
    var upstreamLatency = 75;
    var downstreamLatency = 25;
    var socketServer = mulocal_socket_1.createLocalSocketServer();
    var debugServer = new index_1.MuDebugServer({
        socketServer: socketServer,
        inLatency: upstreamLatency,
        outLatency: downstreamLatency,
    });
    var muServer = new mudb_1.MuServer(debugServer);
    var serverProtocol = muServer.protocol(schema_1.protocolSchema);
    serverProtocol.configure({
        message: {
            ping: function (client, data) {
                if (data === PING_OPCODE) {
                    var delay = Date.now() - timestamp;
                    t.ok(delay > 2 * upstreamLatency, "server should receive opcode after >" + 2 * upstreamLatency + " ms");
                    client.message.pong(PONG_OPCODE);
                }
            },
        },
    });
    muServer.start({
        ready: function () {
            var socket = mulocal_socket_1.createLocalSocket({
                sessionId: randomId(),
                server: socketServer,
            });
            var debugSocket = new index_1.MuDebugSocket({
                socket: socket,
                outLatency: upstreamLatency,
                inLatency: downstreamLatency,
            });
            var muClient = new mudb_1.MuClient(debugSocket);
            var clientProtocol = muClient.protocol(schema_1.protocolSchema);
            clientProtocol.configure({
                message: {
                    pong: function (data) {
                        var delay = Date.now() - timestamp;
                        var latency = upstreamLatency + downstreamLatency;
                        t.equal(data, PONG_OPCODE);
                        t.ok(delay > 2 * latency, "client should receive opcode after >" + 2 * latency + " ms");
                    },
                },
            });
            muClient.start({
                ready: function () {
                    clientProtocol.server.message.ping(PING_OPCODE);
                    timestamp = Date.now();
                },
            });
        },
    });
});
test('LocalSocket - maintaining order of messages from client side', function (t) {
    var head = 0;
    var end = 32;
    t.plan(end - head);
    var socketServer = mulocal_socket_1.createLocalSocketServer();
    var muServer = new mudb_1.MuServer(socketServer);
    var serverProtocol = muServer.protocol(schema_1.protocolSchema);
    serverProtocol.configure({
        message: {
            ping: function (client, data) {
                client.message.pong(data);
            },
        },
    });
    muServer.start({
        ready: function () {
            var socket = mulocal_socket_1.createLocalSocket({
                sessionId: randomId(),
                server: socketServer,
            });
            var debugSocket = new index_1.MuDebugSocket({
                socket: socket,
                outLatency: 75,
                outJitter: 10,
                inLatency: 25,
                inJitter: 10,
            });
            var muClient = new mudb_1.MuClient(debugSocket);
            var clientProtocol = muClient.protocol(schema_1.protocolSchema);
            clientProtocol.configure({
                message: {
                    pong: function (data) {
                        t.equal(data, head++);
                    },
                },
            });
            muClient.start({
                ready: function () {
                    for (var i = head; i < end; ++i) {
                        clientProtocol.server.message.ping(i, false);
                    }
                },
            });
        },
    });
});
test('LocalSocket - maintaining order of messages from server side', function (t) {
    var head = 0;
    var end = 32;
    t.plan(end - head);
    var socketServer = mulocal_socket_1.createLocalSocketServer();
    var debugServer = new index_1.MuDebugServer({
        socketServer: socketServer,
        inLatency: 75,
        inJitter: 10,
        outLatency: 25,
        outJitter: 10,
    });
    var muServer = new mudb_1.MuServer(debugServer);
    var serverProtocol = muServer.protocol(schema_1.protocolSchema);
    serverProtocol.configure({
        message: {
            ping: function (client, data) {
                client.message.pong(data);
            },
        },
    });
    muServer.start({
        ready: function () {
            var socket = mulocal_socket_1.createLocalSocket({
                sessionId: randomId(),
                server: socketServer,
            });
            var muClient = new mudb_1.MuClient(socket);
            var clientProtocol = muClient.protocol(schema_1.protocolSchema);
            clientProtocol.configure({
                message: {
                    pong: function (data) {
                        t.equal(data, head++);
                    },
                },
            });
            muClient.start({
                ready: function () {
                    for (var i = head; i < end; ++i) {
                        clientProtocol.server.message.ping(i, false);
                    }
                },
            });
        },
    });
});
test('LocalSocket - maintaining order of messages from both sides', function (t) {
    var head = 0;
    var end = 32;
    t.plan(end - head);
    var socketServer = mulocal_socket_1.createLocalSocketServer();
    var debugServer = new index_1.MuDebugServer({
        socketServer: socketServer,
        inLatency: 75,
        inJitter: 10,
        outLatency: 25,
        outJitter: 10,
    });
    var muServer = new mudb_1.MuServer(debugServer);
    var serverProtocol = muServer.protocol(schema_1.protocolSchema);
    serverProtocol.configure({
        message: {
            ping: function (client, data) {
                client.message.pong(data);
            },
        },
    });
    muServer.start({
        ready: function () {
            var socket = mulocal_socket_1.createLocalSocket({
                sessionId: randomId(),
                server: socketServer,
            });
            var debugSocket = new index_1.MuDebugSocket({
                socket: socket,
                outLatency: 75,
                outJitter: 10,
                inLatency: 25,
                inJitter: 10,
            });
            var muClient = new mudb_1.MuClient(debugSocket);
            var clientProtocol = muClient.protocol(schema_1.protocolSchema);
            clientProtocol.configure({
                message: {
                    pong: function (data) {
                        t.equal(data, head++);
                    },
                },
            });
            muClient.start({
                ready: function () {
                    for (var i = head; i < end; ++i) {
                        clientProtocol.server.message.ping(i, false);
                    }
                },
            });
        },
    });
});
test('LocalSocket - simulating packet loss on client side', function (t) {
    t.plan(1);
    var socketServer = mulocal_socket_1.createLocalSocketServer();
    var muServer = new mudb_1.MuServer(socketServer);
    var serverProtocol = muServer.protocol(schema_1.protocolSchema);
    serverProtocol.configure({
        message: {
            ping: function (client, data) {
                if (data === PING_OPCODE) {
                    t.pass('server should receive opcode');
                    client.message.pong(PONG_OPCODE);
                }
            },
        },
    });
    muServer.start({
        ready: function () {
            var socket = mulocal_socket_1.createLocalSocket({
                sessionId: randomId(),
                server: socketServer,
            });
            var debugSocket = new index_1.MuDebugSocket({
                socket: socket,
                inPacketLoss: 100,
            });
            var muClient = new mudb_1.MuClient(debugSocket);
            var clientProtocol = muClient.protocol(schema_1.protocolSchema);
            clientProtocol.configure({
                message: {
                    pong: function (data) {
                        if (data === PONG_OPCODE) {
                            t.fail('client should not receive opcode');
                        }
                    },
                },
            });
            muClient.start({
                ready: function () {
                    clientProtocol.server.message.ping(PING_OPCODE);
                },
            });
        },
    });
});
test('LocalSocket - simulating packet loss on server side', function (t) {
    t.plan(1);
    var socketServer = mulocal_socket_1.createLocalSocketServer();
    var debugServer = new index_1.MuDebugServer({
        socketServer: socketServer,
        outPacketLoss: 100,
    });
    var muServer = new mudb_1.MuServer(debugServer);
    var serverProtocol = muServer.protocol(schema_1.protocolSchema);
    serverProtocol.configure({
        message: {
            ping: function (client, data) {
                if (data === PING_OPCODE) {
                    t.pass('server should receive opcode');
                    client.message.pong(PONG_OPCODE);
                }
            },
        },
    });
    muServer.start({
        ready: function () {
            var socket = mulocal_socket_1.createLocalSocket({
                sessionId: randomId(),
                server: socketServer,
            });
            var muClient = new mudb_1.MuClient(socket);
            var clientProtocol = muClient.protocol(schema_1.protocolSchema);
            clientProtocol.configure({
                message: {
                    pong: function (data) {
                        if (data === PONG_OPCODE) {
                            t.fail('client should not receive opcode');
                        }
                    },
                },
            });
            muClient.start({
                ready: function () {
                    clientProtocol.server.message.ping(PING_OPCODE);
                },
            });
        },
    });
});
//# sourceMappingURL=local.js.map